<?php

defined('BASEPATH') OR exit('No direct script access allowed');

class Language extends CI_Controller {

    private $table = "language";
    private $phrase = "phrase";

    public function __construct() {
        parent::__construct();
        $this->load->database();
        $this->load->dbforge();
        $this->load->helper('language');

        $this->auth->check_admin_auth();

        if ($this->session->userdata('user_type') == '2') {
            $this->session->set_userdata(array('error_message' => display('you_are_not_access_this_part')));
            redirect('Admin_dashboard');
        }
    }

    public function index() {
        $data = array(
            'title' => 'Language',
            'languages' => $this->languages()
        );

        $content = $this->parser->parse('language/main', $data, true);
        $this->template->full_admin_html_view($content);
    }

    public function phrase() {
        $data['title'] = 'Phrase Add';
        $data['languages'] = $this->languages();
        $data['phrases'] = $this->phrases1();

        $content = $this->parser->parse('language/phrase', $data, true);
        $this->template->full_admin_html_view($content);
    }

    public function languages() {
        if ($this->db->table_exists($this->table)) {

            $fields = $this->db->field_data($this->table);

            $i = 1;
            foreach ($fields as $field) {
                if ($i++ > 2)
                    $result[$field->name] = ucfirst($field->name);
            }

            if (!empty($result))
                return $result;
        } else {
            return false;
        }
    }

    public function addLanguage() {
        $language = preg_replace('/[^a-zA-Z0-9_]/', '', $this->input->post('language', true));
        $language = strtolower($language);

        if (!empty($language)) {
            if (!$this->db->field_exists($language, $this->table)) {
                $this->dbforge->add_column($this->table, [
                    $language => [
                        'type' => 'TEXT'
                    ]
                ]);
                $this->session->set_flashdata('message', 'Language added successfully');
                redirect('language');
            }
        } else {
            $this->session->set_flashdata('error_message', 'Please try again');
        }
        redirect('Language');
    }

    public function editPhrase($language = null) {
        $data['title'] = 'Phrase Edit';
        $data['language'] = $language;

        #
        #pagination starts
        #
        $config["base_url"] = base_url('Language/editPhrase/' . $language);
        $config["total_rows"] = $this->phrases_count();
        $config["per_page"] = 50;
        $config["uri_segment"] = 4;
        $config["num_links"] = 5;
        /* This Application Must Be Used With BootStrap 3 * */
        $config['full_tag_open'] = "<ul class='pagination'>";
        $config['full_tag_close'] = "</ul>";
        $config['num_tag_open'] = '<li>';
        $config['num_tag_close'] = '</li>';
        $config['cur_tag_open'] = "<li class='disabled'><li class='active'><a href='#'>";
        $config['cur_tag_close'] = "<span class='sr-only'></span></a></li>";
        $config['next_tag_open'] = "<li>";
        $config['next_tag_close'] = "</li>";
        $config['prev_tag_open'] = "<li>";
        $config['prev_tagl_close'] = "</li>";
        $config['first_tag_open'] = "<li>";
        $config['first_tagl_close'] = "</li>";
        $config['last_tag_open'] = "<li>";
        $config['last_tagl_close'] = "</li>";
        /* ends of bootstrap */
        $this->pagination->initialize($config);
        $page = ($this->uri->segment(4)) ? $this->uri->segment(4) : 0;
        $data['links'] = $this->pagination->create_links();
        #
        #pagination ends
        #  

        $data['phrases'] = $this->phrases($config["per_page"], $page);

        $content = $this->parser->parse('language/phrase_edit', $data, true);
        $this->template->full_admin_html_view($content);
    }

    public function addPhrase() {

        $lang = $this->input->post('phrase');

        if (sizeof($lang) > 0) {

            if ($this->db->table_exists($this->table)) {

                if ($this->db->field_exists($this->phrase, $this->table)) {

                    foreach ($lang as $value) {

                        $value = preg_replace('/[^a-zA-Z0-9_]/', '', $value);
                        $value = strtolower($value);

                        if (!empty($value)) {
                            $num_rows = $this->db->get_where($this->table, [$this->phrase => $value])->num_rows();

                            if ($num_rows == 0) {
                                $this->db->insert($this->table, [$this->phrase => $value]);
                                $this->session->set_flashdata('message', 'Phrase added successfully');
                            } else {
                                $this->session->set_flashdata('error_message', 'Phrase already exists!');
                            }
                        }
                    }

                    redirect('Language/phrase');
                }
            }
        }

        $this->session->set_flashdata('error_message', 'Please try again');
        redirect('Language/phrase');
    }

    public function phrases($per_page, $page) {
        if ($this->db->table_exists($this->table)) {

            if ($this->db->field_exists($this->phrase, $this->table)) {

                return $this->db->order_by($this->phrase, 'asc')
                                ->limit($per_page, $page)
                                ->get($this->table)
                                ->result();
            }
        }

        return false;
    }

    public function phrases1() {
        if ($this->db->table_exists($this->table)) {

            if ($this->db->field_exists($this->phrase, $this->table)) {

                return $this->db->order_by($this->phrase, 'asc')
                                ->get($this->table)
                                ->result();
            }
        }

        return false;
    }

    public function phrases_count() {
        if ($this->db->table_exists($this->table)) {
            if ($this->db->field_exists($this->phrase, $this->table)) {
                return $this->db->order_by($this->phrase, 'asc')
                                ->get($this->table)
                                ->num_rows();
            }
        }

        return false;
    }

    public function addLebel() {
        $language = $this->input->post('language', true);
        $phrase = $this->input->post('phrase', true);
        $lang = $this->input->post('lang', true);

        if (!empty($language)) {

            if ($this->db->table_exists($this->table)) {

                if ($this->db->field_exists($language, $this->table)) {

                    if (sizeof($phrase) > 0)
                        for ($i = 0; $i < sizeof($phrase); $i++) {
                            $this->db->where($this->phrase, $phrase[$i])
                                    ->set($language, $lang[$i])
                                    ->update($this->table);
                        }
                    $this->session->set_flashdata('message', 'Label added successfully!');
                    redirect('Language/editPhrase/' . $language);
                }
            }
        }

        $this->session->set_flashdata('error_message', 'Please try again');
        redirect('Language/editPhrase/' . $language);
    }

}
